#!/usr/bin/python
"""Classes to handle sodar data samples

Sodar data samples are collected into daily files. Each sample consists of a
header followed by an observation for each height.
"""

import re

class Data(object):
    """Daily sodar file data (a collection of samples)"""
    def __init__(self,data):
        object.__init__(self)
        # samples in file are terminated by $
        self.samples = [Sample(sample) for sample in [sample.strip() for sample in data.split('$')] if sample]

class Sample(object):
    """A single sample from daily sodar file data (a header and a body)"""
    def __init__(self,sample):
        object.__init__(self)
        # first three groups of lines are the header; rest is body
        samplePattern = re.compile(r'''(?P<header>.*?\n\n.*?\n\n.*?\n\n)
                                       (?P<body>.*$)
                                    ''',re.DOTALL | re.VERBOSE)
        self.__dict__.update(samplePattern.match(sample.strip()).groupdict())
        # fix for missing keys
        self.header = Header(self.header)

class Header(object):
    """A sodar data sample header (a collection of sample-wide parameters)"""
    def __init__(self,header):
        object.__init__(self)
        headerLines = [headerLine.strip() for headerLine in header.split('\n') if headerLine]
        # every other line contains parameter names; every other line contains parameter values
        parametersPairs = [(headerLine,headerLines[headerLines.index(headerLine)+1]) for headerLine in headerLines[::2]]
        for parameterNames,parameterValues in parametersPairs:
            # parameter names must be valid Python identifiers for named groups matching
            parameterNames = [parameterName.strip('#') for parameterName in parameterNames.split()]
            parameterPattern = re.compile(r'(?P<'+'>\S+)\s+(?P<'.join(parameterNames)+'>.*$)')
            self.__dict__.update(parameterPattern.match(parameterValues).groupdict())

def __main():
    """Process as script from command line"""
    import urllib2
    try:
       data = urllib2.urlopen('http://nemo.isis.unc.edu/data/nccoos/level0/dukeforest/sodar/store/2007-06/20070601.dat')
       data = data.read()
    except:
        print "Failure to read test data"
    data = Data(data)
    print '   --- Sample ---\n'.join(data.samples)

if __name__ == "__main__":
    __main()
